//
// Copyright 2021 Vulcanize, Inc.
//

import { Entity, PrimaryColumn, Column, Index } from 'typeorm';
import { OrderStatus, OrderType } from '../types';
import { bigintTransformer } from '@cerc-io/util';

@Entity()
@Index(['blockNumber'])
export class Order {
  @PrimaryColumn('varchar')
    id!: string;

  @PrimaryColumn('varchar', { length: 66 })
    blockHash!: string;

  @Column('integer')
    blockNumber!: number;

  @Column('numeric', { transformer: bigintTransformer })
    orderId!: bigint;

  @Column('varchar')
    user!: string;

  @Column('varchar')
    currency!: string;

  @Column('integer')
    side!: number;

  @Column('numeric', { transformer: bigintTransformer })
    maturity!: bigint;

  @Column('numeric', { transformer: bigintTransformer })
    inputUnitPrice!: bigint;

  @Column('numeric', { transformer: bigintTransformer })
    inputAmount!: bigint;

  @Column('numeric', { transformer: bigintTransformer })
    filledAmount!: bigint;

  @Column({ type: 'enum', enum: OrderStatus })
    status!: OrderStatus;

  @Column('numeric', { transformer: bigintTransformer })
    statusUpdatedAt!: bigint;

  @Column('varchar')
    lendingMarket!: string;

  @Column('boolean')
    isPreOrder!: boolean;

  @Column({ type: 'enum', enum: OrderType })
    type!: OrderType;

  @Column('boolean')
    isCircuitBreakerTriggered!: boolean;

  @Column('numeric', { transformer: bigintTransformer })
    createdAt!: bigint;

  @Column('numeric', { transformer: bigintTransformer })
    _blockNumber!: bigint;

  @Column('varchar')
    txHash!: string;

  @Column('boolean', { default: false })
    isPruned!: boolean;

  @Column('boolean', { default: false })
    isRemoved!: boolean;
}
