//
// Copyright 2021 Vulcanize, Inc.
//

import { Entity, PrimaryColumn, Column, Index } from 'typeorm';
import { TransactionExecutionType } from '../types';
import { decimalTransformer, bigintTransformer } from '@cerc-io/util';
import { Decimal } from 'decimal.js';

@Entity()
@Index(['blockNumber'])
export class Transaction {
  @PrimaryColumn('varchar')
    id!: string;

  @PrimaryColumn('varchar', { length: 66 })
    blockHash!: string;

  @Column('integer')
    blockNumber!: number;

  @Column('varchar')
    currency!: string;

  @Column('numeric', { transformer: bigintTransformer })
    maturity!: bigint;

  @Column('integer')
    side!: number;

  @Column('numeric', { transformer: bigintTransformer })
    executionPrice!: bigint;

  @Column('varchar')
    user!: string;

  @Column({ type: 'enum', enum: TransactionExecutionType })
    executionType!: TransactionExecutionType;

  @Column('numeric', { transformer: bigintTransformer })
    futureValue!: bigint;

  @Column('numeric', { transformer: bigintTransformer })
    amount!: bigint;

  @Column('numeric', { transformer: bigintTransformer })
    feeInFV!: bigint;

  @Column('numeric', { transformer: decimalTransformer })
    averagePrice!: Decimal;

  @Column('varchar')
    lendingMarket!: string;

  @Column('varchar')
    order!: string;

  @Column('numeric', { transformer: bigintTransformer })
    createdAt!: bigint;

  @Column('numeric', { transformer: bigintTransformer })
    _blockNumber!: bigint;

  @Column('varchar')
    txHash!: string;

  @Column('boolean', { default: false })
    isPruned!: boolean;

  @Column('boolean', { default: false })
    isRemoved!: boolean;
}
